{$A+,B-,D+,E+,F-,G+,I+,L+,N+,P+,Q-,R-,S-,T-,V+,X+,Y+}

(* FONTS
	 -----
(C) 1998,99 by Arsne von Wyss - arsene@vonwyss.ch

Rev: 19991207

This source code is freeware for non-commercial use and provided AS-IS.
You are not allowed to use or spread incomplete (inclusive this header)
copies of this code. Use at your own risk. If you do not agree with these
statements, delete this source code immediately. Please contact the
author via email for questions or bug reports.

Source code in development and subject to change frequently. No support from
the author yet.*)

{The Fonts unit provides font objects. Currenty system font (8*8) and TTF are
supported.}

unit Fonts;

{$DEFINE TRUETYPE}

interface

uses
	Canvas{$IFDEF TRUETYPE},FreeType{$ENDIF}; {TrueType support thanks to FreeType, see HTTP://FREETYPE.ORG}

type
	{System font}
	PFont8=^TFont8;
	TFont8=object(TFont)
		Data: array[Char,0..7] of Byte;
		function Width(C: Char): Integer; virtual;
		function Height: Integer; virtual;
		function Out(Canvas: PCanvas; C: Char): Integer; virtual;
		procedure MakeThin;
		procedure MakeBold;
		procedure GrabSystemFont;
	end;
	{$IFDEF TRUETYPE}
	{True Type font, cached hl-maps}
	PHList=^THList;
	THList=array[0..10919] of record
		X1,Y,X2: Integer;
	end;
	PCharPic=^TCharPic;
	TCharPic=object
		Width,Height,Advance,Count: Integer;
		Data: PHList;
		constructor Init(const Glyph: TT_Glyph; const IMetrics: TT_Instance_Metrics);
		procedure Draw(Canvas: PCanvas);
		destructor Done;
	end;
	PTrueTypeFont=^TTrueTypeFont;
	TTrueTypeFont=object(TFont)
	private
		CharMap: array[Char] of PCharPic;
		Glyph: TT_Glyph;
	public
		Face: TT_Face;
		Instance: TT_Instance;
		Metrics: TT_Instance_Metrics;
		constructor Init(const FName: string);
		procedure SetSize(ASize: Integer);
		function Width(C: Char): Integer; virtual;
		function Height: Integer; virtual;
		function Out(Canvas: PCanvas; C: Char): Integer; virtual;
		function GetCharPic(C: Char): PCharPic;
		procedure ClearCharMap;
		destructor Done; virtual;
	end;
	{$ENDIF}

implementation

uses
	GUIUtils,Surfaces{$IFDEF TRUETYPE},TTFile{$ENDIF};

{$IFDEF TRUETYPE}
var
	OpenTTF: Integer;

constructor TTrueTypeFont.Init(const FName: string);
begin
	inherited Init;
	if OpenTTF=0 then
		TT_Init_FreeType;
	Inc(OpenTTF);
	if TT_Open_Face(FName,Face)<>0 then
		Fail;
	TT_New_Glyph(Face,Glyph);
	TT_New_Instance(Face,Instance);
	TT_Set_Instance_Resolutions(Instance,96,96);
	SetSize(7);
end;

procedure TTrueTypeFont.SetSize(ASize: Integer);
begin
	ClearCharMap;
	TT_Set_Instance_CharSize(Instance,ASize*64);
	TT_Get_Instance_Metrics(Instance,Metrics);
end;

function TTrueTypeFont.Width(C: Char): Integer;
begin
	Width:=GetCharPic(C)^.Advance;
end;

function TTrueTypeFont.Height: Integer;
begin
	Height:=Metrics.Y_PPEM;
end;

function TTrueTypeFont.Out(Canvas: PCanvas; C: Char): Integer;
begin
	with GetCharPic(C)^ do begin
		Draw(Canvas);
		Out:=Advance;
	end;
end;

function TTrueTypeFont.GetCharPic(C: Char): PCharPic;
begin
	if not Assigned(CharMap[C]) then begin
		TT_Load_Glyph(Instance,Glyph,Ord(C)-29,TT_Load_Default);
		if C='p' then
			C:='p';
		New(CharMap[C],Init(Glyph,Metrics));
	end;
	GetCharPic:=CharMap[C];
end;

procedure TTrueTypeFont.ClearCharMap;
var
	C: Char;
begin
	for C:=Low(Char) to High(Char) do
		if Assigned(CharMap[C]) then begin
			Dispose(CharMap[C],Done);
			CharMap[C]:=nil;
		end;
end;

destructor TTrueTypeFont.Done;
begin
	ClearCharMap;
	TT_Close_Face(Face);
	Dec(OpenTTF);
	if OpenTTF=0 then
		TT_Done_FreeType;
	inherited Done;
end;

constructor TCharPic.Init(const Glyph: TT_Glyph; const IMetrics: TT_Instance_Metrics);
type
	TSetArray=array[0..1023] of set of 0..7;
	PSetArray=^TSetArray;
var
	Temp: PHList;
	X,YC,I,TempCount,YO,XO: Integer;
	Metrics: TT_Glyph_Metrics;
	Raster: TT_Raster_Map;
begin
	TT_Get_Glyph_Metrics(Glyph,Metrics);
	with Metrics,BBox,Raster do begin
		Width:=(XMax-XMin+63) div 64+1;
		Rows:=(YMax-YMin+63) div 64+1;
		Flow:=TT_Flow_Down;
		Cols:=(Width+7) div 8;
		Size:=Cols*Rows;
		GetMem(Buffer,Size);
		FillChar(Buffer^,Size,0);
		TempCount:=Minimum(High(Temp^)+1,(Succ(Width)*Height) div 2);
		TT_Get_Glyph_Bitmap(Glyph,Raster,-(XMin and (not 63)),-(YMin and (not 63)));
		XO:=(XMin+63) div 64;
		YO:=-(YMax+63) div 64+((IMetrics.Y_PPEM*3) shr 2);
		self.Width:=Width;
		self.Height:=Rows;
	end;
	GetMem(Temp,SizeOf(Temp^[0])*TempCount);
	Count:=0;
	Advance:=Metrics.Advance div 64+1;
	for YC:=0 to Height-1 do begin
		I:=0;
		for X:=0 to Width-1 do
			if not ((7-(X and 7)) in PSetArray(Raster.Buffer)^[X shr 3+YC*Raster.Cols]) then begin
				if I<X then begin
					with Temp^[Count] do begin
						X1:=I+XO;
						X2:=X-1+XO;
						Y:=YC+YO;
					end;
					Inc(Count);
				end;
				I:=X+1;
			end;
		if I<Width-1 then begin
			with Temp^[Count] do begin
				X1:=I+XO;
				X2:=Width-1+XO;
				Y:=YC+YO;
			end;
			Inc(Count);
		end;
	end;
	if Count=0 then
		Data:=nil
	else begin
		GetMem(Data,SizeOf(Temp^[0])*Count);
		Move(Temp^,Data^,SizeOf(Temp^[0])*Count);
	end;
	with Raster do
		FreeMem(Buffer,Size);
	FreeMem(Temp,SizeOf(Temp^[0])*TempCount);
end;

procedure TCharPic.Draw(Canvas: PCanvas);
var
	I: Integer;
begin
	if Assigned(Data) then
		with Canvas^ do begin
			BeginUpdate;
			for I:=0 to Count-1 do
				with Data^[I] do
					if X1=X2 then begin
						if PointIncluded(X1+Cursor.X,Y+Cursor.Y) then
							DirectPutPixel(X1+Cursor.X+Offset.X,Y+Cursor.Y+Offset.Y);
					end else
						HLine(X1+Cursor.X,Y+Cursor.Y,X2+Cursor.X);
			EndUpdate;
		end;
end;

destructor TCharPic.Done;
begin
	if Assigned(Data) then
		FreeMem(Data,SizeOf(Data^[0])*Count);
end;
{$ENDIF}

function TFont8.Width(C: Char): Integer;
var
	I,J,Cnt,Max: Integer;
begin
	Max:=0;
	for I:=0 to 7 do begin
		J:=Data[C,I];
		Cnt:=0;
		while J>0 do begin
			J:=J shr 1;
			Inc(Cnt);
		end;
		if Cnt>Max then
			Max:=Cnt;
	end;
	if C=#32 then
		Inc(Max,4);
	Width:=Max+1;
end;

function TFont8.Height: Integer;
begin
	Height:=8;
end;

function TFont8.Out(Canvas: PCanvas; C: Char): Integer;
var
	X,Y,I,Cnt,Max: Integer;
	J: Byte;
begin
	Max:=0;
	with Canvas^ do begin
		if (C<>#32) and RectOversect(Cursor.X,Cursor.Y,Cursor.X+7,Cursor.Y+7) then begin
			X:=Offset.X+Cursor.X;
			Y:=Offset.Y+Cursor.Y;
			BeginUpdate;
			for I:=0 to 7 do begin
				J:=Data[C,I];
				Cnt:=0;
				while J>0 do begin
					if (J and 1>0) then
						PutPixel(Cursor.X+Cnt,Cursor.Y+I);
					J:=J shr 1;
					Inc(Cnt);
				end;
				if Cnt>Max then
					Max:=Cnt;
			end;
			EndUpdate;
			Out:=Max+1;
		end else
			Out:=self.Width(C);
	end;
end;

{Just for fun...}
procedure TFont8.MakeThin;
var
	C: Char;
	I: Integer;
begin
	for C:=#0 to #255 do
		for I:=0 to 7 do
			Data[C,I]:=Data[C,I] and (Data[C,I] shr 1);
end;

procedure TFont8.MakeBold;
var
	C: Char;
	I: Integer;
begin
	for C:=#0 to #255 do
		for I:=0 to 7 do
			Data[C,I]:=Data[C,I] or (Data[C,I] shl 1);
end;

procedure TFont8.GrabSystemFont;
var
	C: Char;
	I,J,K: Integer;
	Quit: Boolean;
begin
	asm {grab system font}
		push bp
		mov ah,$11
		mov al,$30
		mov bh,3
		int $10
		mov si,bp
		pop bp
		push ds
	{$IFDEF DPMI}
		mov ax,2
		mov bx,$C000
		int $31
	{$ELSE}
		mov ax,es
	{$ENDIF}
		mov ds,ax
		les di,self
		add di,Data
		mov cx,256*8
		cld
	@loop:
		lodsb
		mov bl,8
	@loop2:
		shr al,1
		rcl ah,1
		dec bl
		jnz @loop2
		mov es:[di],ah
		inc di
		loop @loop
		pop ds
	end;
	for C:=#0 to #255 do begin {Align characters to the left}
		Quit:=False;
		repeat
			J:=0;
			for I:=0 to 7 do
				if Data[C,I] and 1>0 then begin
					Quit:=True;
					Break;
				end else
					Inc(J,Ord(Data[C,I]=0));
			if not Quit then begin
				if J=8 then
					Break;
				for I:=0 to 7 do
					Data[C,I]:=Data[C,I] shr 1;
			end;
		until Quit;
	end;
end;

end.