{$A+,B-,D+,E+,F-,G+,I+,L+,N+,P+,Q-,R-,S-,T-,V+,X+,Y+}

(* GUI
	 ---

(C) 1998,1999,2000 by Arsne von Wyss - arsene@vonwyss.ch

Rev: 20000214

This source code is freeware for non-commercial use and provided AS-IS.
You are not allowed to use or spread incomplete (inclusive this header)
copies of this code. Use at your own risk. If you do not agree with these
statements, delete this source code immediately. Please contact the
author via email for questions or bug reports.

GUI: Graphical User Interface - Unit Overview
---------------------------------------------
*)
{ Welcome to the

	GGG U  U I
 G    U  U I
 G GG U  U I
 G  G U  U I
	GGG  UUU I (Graphics User Interface)

 application framework by Arsne von Wyss!

The GUI enables the programmer to implement fully graphics oriented programs
with a "touch and feel" of Windows. In addition, the GUI takes care of many
more or less annoying aspects of user interfaces like text input fields, mouse
checks and clipping when drawing.

The key parts of the GUI are:
 * ~Elements~ like Windows, Buttons and other controls
 * Communication inside the program is done with ~Events~
 * ~TView~s and ~TViewGroups~ handle every aspect of visible controls
 * ~Surfaces~ will keep track shapes and areas, helping to find out over
	 which ~TView~ the mouse is or by holding the clipping information
 * A driver which provides ~Mouse~, ~Screen~, and ~Canvas~ operation

Please report bugs and send your comments or suggestions to the author.
 arsene@vonwyss.ch
Please don't forget to specify the ~revision~ you are talking about!}

unit GUI;

(* Unit interface is HelpGen enabled *)

interface

uses
	Objects,Canvas,Events,Surfaces,Fonts;

const
	{Revision date string. It will always hold the date of the last modification of the
	GUI.

	The format it 'YYMMDD', where YY=00 stands for the year 2000.}
	Revision='990911';
	{ms for double click}
	DoubleClick: Word=250;
	{Command constants for ~TView~s.}
	cmRedraw=256;
	cmClearCanvas=257;
	cmFocusChanged=259;
	cmMouseEnter=260;
	cmMouseLeave=261;
	cmDoubleClick=262;
	cmHandleKey=263;
	cmDesigner=364;

type
	PView=^TView;
	PColorPair=^TColorPair;
	TColorPair=record
		Text,Back: TColor;
	end;
	PColorSet=^TColorSet;
	TColorSet=object(TReferenceObject)
		Frame: record
			Up,Down,Flat: TColor;
		end;
		Normal,Selected,Disabled,Hover: TColorPair;
		Back,Special: TColor;
		function Select(ASelected,AEnabled,AHover: Boolean): PColorPair;
	end;
	{Flag list for ~TView~ resizing flags.}
	TResizeFlag=(rfLeft,rfRight,rfTop,rfBottom);
	{Flag set of resizing flags. These flags will decide how a ~TView~ will be
	moved or resized when its ~parent~ ~TViewGroup~ is ~resize~d. See ~ResizeFlags~
	for details.}
	TResizeFlags=set of TResizeFlag;
	PViewGroup=^TViewGroup;
	{All visible elements will be derived from ~TView~. It provides the basic
	functionality of any element like size, position, ~canvas~, flags and
	pointers to other views, and it provides a large set of manipulation
	and information functions.}
	TView=object(TSurface)
	private
		Initialized,WasInFocus: Boolean;
		OldModal: PView;
		CanvasShape: TSurface;
		Designer: PEventObject;
		procedure RefreshCovered(X1,Y1,X2,Y2: Integer);
	public
		Background: PBitmap;
		{The two fileds of the TRect structure define the minimum and the maximum
		size of this view. By default these settings are set to (0,0) and
		(MaxInt,MaxInt), making about any size possible. These fields can be directly
		accessed, but note that they will only apply from the next ~Resize~ on.

		TWindow will set a minimum size in order not to disturb any of the views
		inside it when resizing (see ~ResizeFlags~ for more information).}
		SizeLimit: TRect;
		{~Left~ and ~Top~ are the current position relative to its ~parent~
		~TViewGroup~. You should not write to these values but use ~Move~ instead.}
		Left,Top: Integer;
		{This points to the owner ~TViewGroup~ holding this view as a child. The
		Owner will make sure that this ~TView~ will be handled correclty, including
		automatic updates to the ~Canvas~, resized according to the ~ResizeFlags~ and
		correclt ~show~n and ~redraw~n. Do not write directly to the Owner method but
		use the ~InsertToGroup~ and ~RemoveFromGroup~ procedures to make changes.}
		Owner: PViewGroup;
		{Points to the ~next~ and ~prev~ios ~TView~ in the same ~TViewGroup~. If the
		parent is not assigned, these will be NIL.}
		Next,Prev: PView;
		{The ~TEventObject~ that will handle the command events for this view. Note
		that the ~HandleEvent~ of the view will resent the event to the assigned
		EventHandler. Therefore an inherited ~HandleEvent~ will be able to intercept
		or modify messages before they are sent to the EventHandler. If nil, it will
		not be resent. Note that you must avoid deadcircles (EventHandler pointing
		directly or indirectly to the same ~TView~).}
		EventHandler: PEventHandle;
		{Visibility information. This does not refelct if the view is really visible
		on the screen, it may be covered or one of its ~owner~s might be inivisble.}
		Visible,
		{Flag that will make a ~TView~ stand on top of the others in a ~TViewGroup~.
		This means that ~MoveUp~ and ~MoveToTop~ of other group members will not move
		this ~TView~ down. It will also prevent the View of bein put behind other
		group memebers without AlwaysOnTop set when using ~MoveDown~ and ~MoveToBottom~.}
		AlwaysOnTop,
		{Using AcceptFocus a ~TView~ can avoid receiving the focus. In fact, most
		~TView~s will not accept the focus, so this is the default value.
		Note that some ~TView~s will "pretend" to accept the focus but pass it on to
		another ~TView~ by means of the ~ViewToFocus~ method.}
		AcceptFocus,
		{Information flag that will show if the ~TView~ is enabled or not. This flag
		does not affect static views and it can be used for any desired action.

		Use ~Enable~ and ~Disable~ to change the flags state.}
		Enabled,
		{Read only flag that allows to check if a ~TView~ descandant is actually a
		~TViewGroup~ and can safely by typecasted.}
		IsGroup: Boolean;
		{The ResizeFlags decide what happens to the view if its ~owner~ ~resize~s.
		The flags rfLeft,rfRight,rfTop and rfBottom decide, which borders of the view
		will follow the resizment.

		To make the ~TView~ grow with its parent, use [rfRight,rfBottom].

		To make the ~TView~ right aligned, use [rfLeft,rfRight].

		To make the ~TView~ bottom aligned, use [rfTop,rfBottom].}
		ResizeFlags: TResizeFlags;
		{Read only. The Canvas is the drawing surface of the ~TView~. You have to
		access the canvas' properties in order to display the component in the
		~Draw~ method.

		The canvas also holds some useful information such as the ~offset~ of the
		view relative to the screen. ~SurfaceCount~ and ~SurfaceRect~ can provide
		useful information when drawing complex shapes since they hold the list
		of visible rectangles on the screen.
		

		with Canvas^ do begin
			GotoXY(10,10);
			SetColor(clBlack);
			WriteLn(Text,'Line number ',LineNr);
		end;}
		Canvas: PCanvas;
		{The Tag is a freely available variable for any use. You may use it to
		distinguish components or to hold other information. Since it is a Longint
		you can also typecast it to a pointer if necessary to make it hold more
		information.}
		Tag: Longint;
		{}
		ColorSet: PColorSet;
		{Initializes the ~TView~ and inserts it to the given ~TViewGroup~. The view
		will be hidden and of more or less undefined size (the initial size of a
		~TView~ is (0,0) but some descendants resize it already). You should use
		~Resize~, ~Move~ and ~Show~ to make the TView visible like you want it to be.}
		constructor Init(AOwner: PViewGroup);
		{Initialize is called on the first Show call of a view. It s not guaranteed
		that the owner already has called Initialize. You can override Initialize
		to make post-initalizations such as loading data into a component.}
		procedure Initialize; virtual;
		{Refreshes the ~TView~s ~Canvas~. RefreshCanvas is usually called automatically
		when necessary. However, if you modify the ~CanvasShape~ you should call the
		~owner~s RefreshCanvas to reflect the changes.

		~TViewGroup.RefreshCanvas~ also refreshes all its childrens.}
		procedure RefreshCanvas; virtual;
		{}
		procedure SetDesigner(ADesigner: PEventObject); virtual;
		{Frees all memory used by the ~TView~ and destroy the object.

		~TViewGroup.Done~ will also free all its children.}
		destructor Done; virtual;
		{Moves the ~TView~ to the specified coordinates relative to its owner.

		If necessary, ~Draw~ will be called automatically through ~Redraw~ for an update
		of the newly visible parts of the ~TView~.}
		procedure Move(ALeft,ATop: Integer); virtual;
		{Resizes the ~TView~ to the specified size.

		If necessary, ~Draw~ will be called automatically through ~Redraw~ for an update
		of the newly visible parts of the ~TView~.}
		procedure Resize(AWidth,AHeight: Integer); virtual;
		{Hides the ~TView~. If the ~TView~ was modal it will return the modal state to the
		previously modal ~TView~ if any.}
		procedure Hide; virtual;
		{Shows the ~TView~. To show a ~TView~ in a modal way that is no other view below
		will receive any mouse or keyboard events and cannot be focused, making the view
		block any user action not applied on this ~TView~.}
		procedure Show; virtual;
		{Moves the ~TView~ up in the Z order of the owner. Note that an enabled
		~AlwaysOnTop~-Flag on other ~TView~s can prevent MoveUp of making any change.}
		procedure MoveUp;
		{Moves the ~TView~ up in the Z order of the owner. Note that an enabled
		~AlwaysOnTop~-Flag on this ~TView~ can prevent MoveDown of making any change.}
		procedure MoveDown;
		{Moves the ~TView~ up to the top in the Z order of the owner. Note that
		an enabled ~AlwaysOnTop~-Flag on this ~TView~ can prevent MoveDown of
		making any change.}
		procedure MoveToTop;
		{Moves the ~TView~ down to the bottom in the Z order of the owner. Note that
		an enabled ~AlwaysOnTop~-Flag on this ~TView~ can prevent MoveDown of
		making any change.}
		procedure MoveToBottom;
		{Inserts the ~TView~ to a group or moves it to another group.}
		procedure InsertToGroup(AOwner: PViewGroup);
		{Removes the ~TView~ from the current group. Note that you'll have to take
		care of freeing the ~TView~'s memory as long as it is not reinserted to a
		new group using ~IndertToGroup~.}
		procedure RemoveFromGroup;
		{Gets the input capture. From now on to the call of ReleaseCapture all mouse
		and keybaord events are sent to this ~TView~. Note that some global available
		information such as ~ViewUnterMouse~ will not be updates as long as the capture
		os enabled.}
		procedure GetCapture;
		{Releases a previously requested input capture using ~GetCapture~ or
		~GetDragCapture~, making events available to all ~TView~s again.}
		procedure ReleaseCapture;
		{Gets the inpute capture in drag mode. Everything works like when using
		~GetCapture~ except that all mouse messages will be made relative to the
		specified point, making this mode useful for actions relative to a certain
		position of the ~TView~ such a moving or resizing.}
		procedure GetDragCapture(const StartPoint: TPoint);
		{Put the focus on the TView. Note that the focus may no be accepeted by the
		~TView~ depending of the state of ~AcceptFocus~ and ~ViewToFocus~.}
		procedure GetFocus;
		{Drops the focus to the next or previous ~TView~s or is parent. It is
		guaranteed the the focus will be dropped.}
		procedure DropFocus(Backup: Boolean);
		{Enables the ~TView~. See also ~Enabled~ and ~Disable~.}
		procedure Enable; virtual;
		{Disables the ~TView~. See also ~Enabled~ and ~Enable~.}
		procedure Disable; virtual;
		{Redraws the ~TView~ using the given update surface. If no surface is
		given the whole ~TView~ is redrawn.}
		procedure Redraw(ASurface: PSurface); virtual;
		{Move the ~Canvas~' offset. Do not call directly.}
		procedure MoveOffset(RelX,RelY: Integer); virtual;
		{~Show~s the ~TView~ in modal mode.}
		procedure ShowModal;
		{IsModal will return TRUE is this ~TView~ in in a modal state or group.}
		function IsModal: Boolean;
		{TRUE if the ~TView~ currently has the focus.}
		function Focused: Boolean;
		{TRUE if the ~TView~ currently has the focus or if the ~TView~ is a group
		holding the ~focused~ ~TView~.}
		function InFocus: Boolean;
		{TRUE if ~Focused~ has just changed.}
		function FocusedChanged: Boolean;
		{TRUE if ~InFocus~ has just changed.}
		function InFocusChanged: Boolean;
		{TRUE is the mouse is currently over the component or if is has the input
		capture.}
		function UnderMouse: Boolean;
		{}
		function IsInGroup(AGroup: PViewGroup): Boolean;
		{TRUE if it has the input capture.}
		function HasCapture: Boolean;
		{Searches for the view under the specified position. Note that a ~TView~
		will always return itself or NIL as result. This function becomes useful
		in ~TViewGroup~s where it also searhec among its childs.}
		function FindView(X,Y: Integer): PView; virtual;
		{Draws the ~TView~. This is an abstract method so any ~TView~ descendant
		must implement it. You can assume that Draw is only called if the view
		is visible on the screen, so you don't have to worry about checking the
		~TView~s state. Inside the Draw method you can draw to the given ~Canvas~.

		However, if you want to refresh a ~TView~, do not call Draw directly but
		instead use Redraw(nil) which will take care of the necessary checks.}
		procedure Draw; virtual;
		{The ~TView~s event handler, inherited from ~TEventObj.HandleEvent~.

		If you overwrite it, make sure to call the inherited method.}
		procedure HandleEvent(var Event: TEvent); virtual;
		{Function that returns the view that should be focused when this ~TView~
		is in the process of receiving the focus. Usually the function returns
		a pointer itself, but sometimes a different behavoiour when focusing is
		desirable.

		Note that ~AcceptFocus~ is stroger than ViewToFocus.}
		function ViewToFocus: PView; virtual;
		{Returns true is all the necessary condition are fulfilled that make a
		~TView~ drawable.}
		function CanDraw: Boolean;
		procedure FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer); virtual;
		procedure SetBackground(ABack: PBitmap);
		procedure ClearBackground;
		procedure ChangeColorSet(AColorset: PColorSet);
		procedure UniqueColorSet;
		procedure SetEventHandler(AHandle: PEventHandle);
	end;
	{Any ~TView~ derived from ~TViewGroup~ will be able to have children views
	inserted. The view group will take care of all aspects needed for correct
	word of the child views. There is always at least one of these available,
	that is, the ~screen~.}
	TViewGroup=object(TView)
		{read only}Child,{read only}FocusChild: PView;
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
		procedure RefreshCanvas; virtual;
		procedure RefreshOwnCanvas;
		destructor Done; virtual;
		procedure Redraw(ASurface: PSurface); virtual;
		procedure MoveOffset(RelX,RelY: Integer); virtual;
		procedure Resize(AWidth,AHeight: Integer); virtual;
		function FindView(X,Y: Integer): PView; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
		function InsertionGroup: PViewGroup; virtual;
		procedure SetDesigner(ADesigner: PEventObject); virtual;
	end;
	PScreen=^TScreen;
	{The main screen. The driver unit needs to inherite it to create the
	right canvas objects for the views. The screen is always available
	when the application is running. For details on driver units check
	out the example driver unit.}
	TScreen=object(TViewGroup)
	private
		OldFocus: PView;
		Dragging,Updating: Boolean;
		DraggingOffset: TPoint;
		procedure Update;
	public
		AutoFocus: Boolean;
		{read only}Focus,
		{read only}Capture,
		{read only}ViewUnderMouse,
		{read only}Modal: PView;
		{read only}OriginalOutput: Text;
		Tile: Boolean;
		BitmapFormat: TBitmapFormat;
		constructor Init(AWidth,AHeight: Integer);
		procedure SetFocus(AView: PView);
		procedure Hide; virtual;
		procedure Show; virtual;
		procedure Draw; virtual;
		function NewCanvas: PCanvas; virtual; {ABSTRACT}
		procedure HandleEvent(var Event: TEvent); virtual;
		procedure ConvertBitmap(ABitmap: PBitmap); virtual;
		procedure FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer); virtual;
	end;
	PApplication=^TApplication;
	PMouse=^TMouse;
	{Mouse object; needs to be filled out by driver. Check the driver unit
	supplied for more info.}
	TMouse=object(TObject)
	private
		LastKey: Integer;
		LastKeyTime: Longint;
	public
		{read only}Keys: Integer;
		{read only}Position: TPoint;
		constructor Init;
		procedure Update(X,Y,Key: Integer); virtual;
		procedure IntUpdate(X,Y,Key: Integer);
		procedure Show; virtual; {ABSTRACT}
		procedure Hide; virtual; {ABSTRACT}
		procedure Limit(X1,Y1,X2,Y2: Integer); virtual; {ABSTRACT}
		procedure Move(X,Y: Integer); virtual; {ABSTRACT}
	end;
	TEventPoster=object(TEventBuilder)
		procedure OpenEvent(var Event: PEvent); virtual;
		procedure Mouse(Kind: Word; const AWhere: TPoint; AButtons: Byte; ADouble: Boolean);
		procedure Keyboard(Kind: Word; ACharCode: Char; AScanCode: Byte);
		procedure Timer(Delay: Longint; AHandle: PEventHandle; ACommand: Word; ATag: Longint);
	end;
	TEventSender=object(TEventBuilder)
		procedure OpenEvent(var Event: PEvent); virtual;
		procedure CloseEvent(Event: PEvent); virtual;
	end;
	{The programs using the GUI will inherite from the application object.}
	PDriver=^TDriver;
	TApplication=object(TEventObject)
	private
		Events: TEventQueue;
	public
		{read only}Running: Boolean;
		{read only}Post: TEventPoster;
		{read only}Send: TEventSender;
		{read only}LastInputEvent: Longint;
		constructor Init(ADriver: PDriver);
		procedure Run; virtual;
		procedure GenerateEvents; virtual;
		function IdleTime: Longint;
		destructor Done; virtual;
		procedure Halt;
		procedure Idle; virtual;
	end;
	{Object to be inherited by the driver unit.}
	TDriver=object(TObject)
		procedure Open; virtual; {ABSTRACT}{}
		procedure Check; virtual;
		procedure Close; virtual;
	end;

var
	{Gives access to the screen and his views.}
	Screen: PScreen;
	{Gives access to the mouse, positions are screen-absolute.}
	Mouse: PMouse;
	{Gives access to the current application.}
	Application: PApplication;
	{The default font used in new canvas objects. You may change it whenever
	you want, but it MUST point to a valid font.}
	SystemFont: PFont;
	{Timer provided by the driver unit in milliseconds.}
	Timer: function: Longint;
	{The driver must be assigned with a TDriver descandant of a driver unit proir
	to the GUI initialization through ~TApplication.Init~.}
	Driver: PDriver;
	{$IFDEF DEBUG}
	Views,MaxViews: Integer;
	{$ENDIF}

implementation

uses
	GUIUtils,CanvasFX;

type
	PTimer=^TTimer;
	TTimer=object(TObject)
		Due: Longint;
		Next: PTimer;
		ViewHandle: PEventHandle;
		Command: Word;
		Tag: Longint;
		constructor Init(Delay: Longint; AHandle: PEventHandle; ACommand: Word; ATag: Longint);
		destructor Done; virtual;
	end;

var
	TimerQueue: PTimer;
	TimerCnt: Integer;

function TColorSet.Select(ASelected,AEnabled,AHover: Boolean): PColorPair;
begin
	if AEnabled then
		if ASelected then
			Select:=@Selected
		else if AHover then
			Select:=@Hover
		else
			Select:=@Normal
	else
		Select:=@Disabled;
end;

{* TView *}

constructor TView.Init(AOwner: PViewGroup);
begin
	inherited Init(0,0,False);
	CanvasShape.Init(0,0,False);
	if not Assigned(Screen) then
		RunError(0); {No Screen???}
	Canvas:=Screen^.NewCanvas;
	Canvas^.Font:=SystemFont;
	if Assigned(AOwner) then
		InsertToGroup(AOwner);
	SizeLimit.B.X:=32767;
	SizeLimit.B.Y:=32767;
	Enabled:=True;
{$IFDEF DEBUG}
	Inc(Views);
	if Views>MaxViews then
		MaxViews:=Views;
{$ENDIF}
end;

procedure TView.HandleEvent(var Event: TEvent);
var
	EventType: Integer;
begin
	with Event do begin
		case What of
		evCommand,evBroadcast:
			case Command of
			cmRedraw: begin
					if Visible then
						Draw;
					EventHandled(Event);
				end;
			cmClearCanvas: Canvas^.SurfaceClear(False);
			end;
		evMouseDouble: if Assigned(EventHandler) then
				Application^.Send.Command(EventHandler,cmDoubleClick,Longint(@self));
		end;
		if Assigned(EventHandler) and (What in [evCommand,evBroadcast]) then begin
			EventType:=What;
			What:=evCommand;
			EventHandler^.Event(Event);
			if Event.What=evCommand then
				What:=EventType;
		end;
	end;
end;

procedure TView.Initialize;
begin
	{Nothing to do here...}
end;

procedure TView.Draw;
begin
	{Nothing to do here...}
end;

procedure TView.Move(ALeft,ATop: Integer);
var
	I,J,K,X1,Y1,X2,Y2,BX1,BY1,BX2,BY2: Integer;
	Back: PBitmap;
	ToDraw,ToCopy,Temp: TSurface;
	R: TRect;
	L: Longint;
begin
	if (ALeft=Left) and (ATop=Top) then
		Exit;
	FindBackground(Back,BX1,BY1);
	Dec(ALeft,Left);
	Dec(ATop,Top);
	Inc(Left,ALeft);
	Inc(Top,ATop);
	MoveOffset(ALeft,ATop);
	if Assigned(Owner) then begin
		if Visible then begin
			with Screen^ do
				if (Capture=@self) and Dragging then begin
					Dec(DraggingOffset.X,ALeft);
					Dec(DraggingOffset.Y,ATop);
				end;
			ToDraw.Init(Owner^.Width,Owner^.Height,False);
			ToDraw.SurfaceCombine(@CanvasShape,Left-ALeft,Top-ATop,cmOR,False);
			ToCopy.InitCopy(@CanvasShape);
			Owner^.RefreshOwnCanvas;
			RefreshCanvas;
			if ALeft>0 then begin
				X1:=Left-ALeft;
				X2:=Left+Width-1;
			end else begin
				X1:=Left;
				X2:=Left+Width-1-ALeft;
			end;
			if ATop>0 then begin
				Y1:=Top-ATop;
				Y2:=Top+Height-1;
			end else begin
				Y1:=Top;
				Y2:=Top+Height-1-ATop;
			end;
			RefreshCovered(X1,Y1,X2,Y2);
			ToCopy.SurfaceCombine(@CanvasShape,0,0,cmAND,False);
			ToDraw.SurfaceCombine(@CanvasShape,Left,Top,cmOR,False);
			FindBackground(Back,BX2,BY2);
			if (ToCopy.SurfaceCount>0) and ((not Assigned(Back)) or ((BX1=BX2) and (BY1=BY2))) then begin
				ToDraw.SurfaceCombine(@ToCopy,Left,Top,cmANDNOT,False);
				with ToCopy do begin
					J:=SurfaceRect^[0].A.Y;
					I:=1;
					repeat
						while (I<SurfaceCount-1) and (SurfaceRect^[I].A.Y=J) do
							Inc(I);
						J:=SurfaceRect^[I].A.Y;
						SurfaceRectHLineCut(J);
						Inc(I);
					until I>=SurfaceCount;
				end;
				if (ALeft<0) xor (ATop<=0) then begin {swap X-Coordinates of blocks}
					I:=0;
					with ToCopy do
						repeat
							J:=SurfaceRect^[I].A.Y;
							K:=Succ(I);
							while (K<SurfaceCount) and (SurfaceRect^[K].A.Y=J) do
								Inc(K);
							J:=K-1;
							while I<J do begin
								Swap(SurfaceRect^[J],SurfaceRect^[I],SizeOf(TRect));
								Inc(I);
								Dec(J);
							end;
							I:=K;
						until I>=SurfaceCount;
				end;
				Temp.Init(Canvas^.Width,Canvas^.Height,True);
				Temp.SurfaceSwap(Canvas);
				Canvas^.BeginUpdate;
				if (ATop<=0) then
					for I:=0 to ToCopy.SurfaceCount-1 do
						with ToCopy.SurfaceRect^[I] do
							Canvas^.Blit(A.X-ALeft,A.Y-ATop,B.X-ALeft,B.Y-ATop,A.X,A.Y)
				else
					for I:=ToCopy.SurfaceCount-1 downto 0 do
						with ToCopy.SurfaceRect^[I] do
							Canvas^.Blit(A.X-ALeft,A.Y-ATop,B.X-ALeft,B.Y-ATop,A.X,A.Y);
				Temp.SurfaceSwap(Canvas);
				Temp.Done;
				if ToDraw.SurfaceCount>0 then
					Owner^.Redraw(@ToDraw);
				Canvas^.EndUpdate;
			end else if ToDraw.SurfaceCount>0 then begin
				Canvas^.BeginUpdate;
				Owner^.Redraw(@ToDraw);
				Canvas^.EndUpdate;
			end;
			ToCopy.Done;
			ToDraw.Done;
		end;
	end;
end;

procedure TView.Resize(AWidth,AHeight: Integer);
var
	OldW,OldH: Integer;
	BackDraw: TSurface;
begin
	if AWidth<SizeLimit.A.X then
		AWidth:=SizeLimit.A.X;
	if AHeight<SizeLimit.A.Y then
		AHeight:=SizeLimit.A.Y;
	if AWidth>SizeLimit.B.X then
		AWidth:=SizeLimit.B.X;
	if AHeight>SizeLimit.B.Y then
		AHeight:=SizeLimit.B.Y;
	if (AWidth<>Width) or (AHeight<>Height) then begin
		OldW:=Width;
		OldH:=Height;
		SurfaceResize(AWidth,AHeight,True);
		if Assigned(Owner) and Visible then begin
			BackDraw.Init(Owner^.Width,Owner^.Height,False);
			BackDraw.SurfaceCombine(@CanvasShape,Left,Top,cmOR,False);
			Owner^.RefreshCanvas;
			RefreshCovered(Left,Top,Left+Maximum(OldW,AWidth)-1,Top+Maximum(OldH,AHeight)-1);
			BackDraw.SurfaceCombine(@CanvasShape,Left,Top,cmOR,False);
			Owner^.Redraw(@BackDraw);
			BackDraw.Done;
		end;
	end;
end;

procedure TView.Hide;
var
	ToDraw: TSurface;
	Temp: PView;
begin
	if Visible and Assigned(Owner) then begin
		Temp:=Screen^.Modal;
		if Temp=@self then begin
			Screen^.Modal:=OldModal;
			OldModal:=nil;
		end else
			while Assigned(Temp) do begin
				if Temp^.OldModal=@self then begin
					Temp^.OldModal:=OldModal;
					Break;
				end;
				Temp:=Temp^.OldModal;
			end;
		OldModal:=nil;
		Visible:=False;
		Application^.Send.Broadcast(Handle,cmClearCanvas,0);
		DropFocus(True);
		if HasCapture then
			ReleaseCapture;
		Owner^.RefreshCanvas;
		ToDraw.Init(Owner^.Width,Owner^.Height,False);
		ToDraw.SurfaceCombine(@CanvasShape,Left,Top,cmOR,False);
		Owner^.Redraw(@ToDraw);
		ToDraw.Done;
		Screen^.Update;
	end;
end;

procedure TView.Show;
begin
	if Assigned(Owner) and (not Visible) then begin
		if not Initialized then begin
			Initialize;
			Initialized:=True;
		end;
		Visible:=True;
		Owner^.RefreshCanvas;
		RefreshCovered(Left,Top,Left+Width-1,Top+Height-1);
		Redraw(nil);
		Screen^.Update;
	end;
end;

procedure TView.MoveUp;
var
	ToDraw: TCanvas;
begin
	if Assigned(Prev) and (Prev^.AlwaysOnTop<=AlwaysOnTop) then begin
		if Assigned(Next) then
			Next^.Prev:=Prev;
		Prev^.Next:=Next;
		Next:=Prev;
		Prev:=Prev^.Prev;
		Next^.Prev:=@self;
		if Assigned(Prev) then
			Prev^.Next:=@self
		else
			Owner^.Child:=@self;
		if Visible then begin
			ToDraw.InitCopyInvert(@CanvasShape);
			RefreshCanvas;
			Next^.RefreshCanvas;
			ToDraw.SurfaceCombine(@CanvasShape,0,0,cmAND,False);
			Redraw(@ToDraw);
			ToDraw.Done;
		end;
	end;
end;

procedure TView.MoveDown;
begin
	if Assigned(Next) and (AlwaysOnTop<=Next^.AlwaysOnTop) then
		Next^.MoveUp;
end;

procedure TView.MoveToTop;
begin
	while Assigned(Prev) and (Prev^.AlwaysOnTop<=AlwaysOnTop) do
		MoveUp;
end;

procedure TView.MoveToBottom;
begin
	while Assigned(Next) and (AlwaysOnTop<=Next^.AlwaysOnTop) do
		MoveDown;
end;

procedure TView.SetDesigner(ADesigner: PEventObject);
begin
	Designer:=ADesigner;
end;

procedure TView.GetFocus;
begin
	if Visible and (not Focused) then
		Screen^.SetFocus(@self);
end;

procedure TView.DropFocus(Backup: Boolean);
var
	Candidate: PView;
begin
	if Backup then begin
		Candidate:=Next;
		while Assigned(Candidate) and (not (Candidate^.Visible and Candidate^.AcceptFocus)) do
			Candidate:=Candidate^.Next;
		if (not Assigned(Candidate)) and Assigned(Owner) then begin
			Candidate:=Owner^.Child;
			if not Assigned(Candidate) then
				Exit;
			while (Candidate<>@self) and (not (Candidate^.Visible and Candidate^.AcceptFocus)) do
				Candidate:=Candidate^.Next;
		end;
	end else begin
		Candidate:=Prev;
		while Assigned(Candidate) and (not (Candidate^.Visible and Candidate^.AcceptFocus)) do
			Candidate:=Candidate^.Prev;
		if not Assigned(Candidate) then begin
			Candidate:=Next;
			if Assigned(Candidate) then begin
				while Assigned(Candidate^.Next) do
					Candidate:=Candidate^.Next;
				while (Candidate<>@self) and (not (Candidate^.Visible and Candidate^.AcceptFocus)) do
					Candidate:=Candidate^.Prev;
			end;
		end;
	end;
	if (not Assigned(Candidate)) or (Candidate=@self) then
		Candidate:=Owner;
	if InFocus then begin
		Screen^.SetFocus(Candidate);
		if InFocus then
			Screen^.SetFocus(nil); {the focus has to be dropped!}
	end else if Assigned(Owner) and (Owner^.FocusChild=@self) then
		Owner^.FocusChild:=Candidate;
end;

procedure TView.Enable;
begin
	Enabled:=True;
	if Visible then
		Draw;
end;

procedure TView.Disable;
begin
	Enabled:=False;
	if Visible then
		Draw;
end;

function TView.ViewToFocus: PView;
begin
	if Enabled then
		ViewToFocus:=@self
	else
		ViewToFocus:=Owner;
end;

function TView.Focused: Boolean;
begin
	Focused:=Screen^.Focus=@self;
end;

function TView.InFocus: Boolean;
var
	Act: PView;
begin
	Act:=Screen^.Focus;
	while Assigned(Act) and (Act<>@self) do
		Act:=Act^.Owner;
	InFocus:=Assigned(Act) and (Act<>PView(Screen));
end;

function TView.FocusedChanged: Boolean;
begin
	with Screen^ do
		FocusedChanged:=(Focus=@self) or (OldFocus=@self);
end;

function TView.InFocusChanged: Boolean;
var
	InNow: Boolean;
begin
	InNow:=InFocus;
	InFocusChanged:=InNow<>WasInFocus;
	WasInFocus:=InNow;
end;

procedure TView.InsertToGroup(AOwner: PViewGroup);
var
	Act: PView;
begin
	Act:=nil;
	while Assigned(AOwner) and (Act<>PView(AOwner)) do begin
		Act:=AOwner;
		AOwner:=AOwner^.InsertionGroup;
	end;
	if (not Assigned(AOwner)) or (PView(AOwner)=@self) then
		RunError(0); {Cannot insert to nil}
	if Owner=AOwner then
		Exit;
	Visible:=False;
	Initialized:=False;
	RemoveFromGroup;
	Next:=AOwner^.Child;
	Prev:=nil;
	if Assigned(Next) then
		Next^.Prev:=@self;
	AOwner^.Child:=@self;
	Owner:=AOwner;
	if Assigned(ColorSet) then
		ColorSet^.Free;
	ColorSet:=Owner^.ColorSet;
	if not Assigned(ColorSet) then begin
		ColorSet:=Screen^.ColorSet;
		if not Assigned(ColorSet) then
			RunError;
	end;
	ColorSet^.Allocate;
	Designer:=Owner^.Designer;
	with Owner^.Canvas^.Offset do
		MoveOffset(X,Y);
	Owner^.RefreshCanvas;
	if not AlwaysOnTop then
		while Assigned(Next) and (Next^.AlwaysOnTop) do
			MoveDown;
	RefreshCovered(Left,Top,Left+Width-1,Top+Height-1);
end;

procedure TView.RemoveFromGroup;
begin
	if not Assigned(Owner) then
		Exit;
	Hide;
	with Owner^.Canvas^.Offset do
		MoveOffset(-X,-Y);
	if Assigned(Prev) then
		Prev^.Next:=Next
	else
		Owner^.Child:=Next;
	if Assigned(Next) then
		Next^.Prev:=Prev;
	Owner:=nil;
end;

procedure TView.ShowModal;
var
	Temp: PView;
begin
	if not Visible then begin
		Temp:=@self;
		while Assigned(Temp) and (Temp<>PView(Screen)) do
			Temp:=Temp^.Owner;
		if not Assigned(Temp) then
			RunError; {MakeModal on component not on screen}
		Show;
		GetFocus;
{		if not InFocus then
			RunError;}
		OldModal:=Screen^.Modal;
		Screen^.Modal:=@self;
	end;
end;

function TView.IsModal: Boolean;
var
	Temp: PView;
begin
	if Assigned(Screen^.Modal) then begin
		Temp:=@self;
		while Assigned(Temp) do begin
			if (Temp=Screen^.Modal) then begin
				IsModal:=True;
				Exit;
			end;
			Temp:=Temp^.Owner;
		end;
	end;
	IsModal:=False;
end;

function TView.IsInGroup(AGroup: PViewGroup): Boolean;
var
	Act: PViewGroup;
begin
	Act:=Owner;
	while Assigned(Act) and (Act<>AGroup) do
		Act:=Act^.Owner;
	IsInGroup:=Assigned(Act);
end;

procedure TView.GetCapture;
begin
	Screen^.Capture:=@self;
	Screen^.Dragging:=False;
end;

procedure TView.GetDragCapture(const StartPoint: TPoint);
begin
	GetCapture;
	Screen^.Dragging:=True;
	Screen^.DraggingOffset:=StartPoint;
end;

procedure TView.ReleaseCapture;
begin
	Screen^.Capture:=nil;
	Screen^.Update;
end;

function TView.HasCapture: Boolean;
begin
	HasCapture:=Screen^.Capture=@self;
end;

function TView.UnderMouse: Boolean;
begin
	UnderMouse:=Screen^.ViewUnderMouse=@self;
end;

function TView.FindView(X,Y: Integer): PView;
begin
	if CanvasShape.PointIncluded(X,Y) then
		FindView:=@self
	else
		FindView:=nil;
end;

procedure TView.Redraw(ASurface: PSurface);
var
	RealCanvas: TSurface;
begin
	if Visible and (Canvas^.SurfaceCount>0) then
		if Assigned(ASurface) then begin
			RealCanvas.InitCopy(Canvas);
			Canvas^.SurfaceCombine(ASurface,0,0,cmAND,False);
			if Canvas^.SurfaceCount>0 then begin
				Canvas^.BeginUpdate;
				Draw;
				Canvas^.EndUpdate;
			end;
			{Canvas^.Clear; (* show redraw regions *)}
			RealCanvas.SurfaceSwap(Canvas);
			RealCanvas.Done;
		end else begin
			Canvas^.BeginUpdate;
			Draw;
			Canvas^.EndUpdate;
		end;
end;

procedure TView.RefreshCanvas;
var
	Act: PView;
begin
	with Canvas^ do
		if Visible then begin
			SurfaceCopy(@self);
			if Assigned(Owner) then begin
				SurfaceCombine(@Owner^.CanvasShape,-Left,-Top,cmAND,True);
				Act:=Prev;
				while Assigned(Act) do begin
					with Act^ do
						if Visible and (Width>0) and (Height>0) then
							self.Canvas^.SurfaceCombine(Act,Left-self.Left,Top-self.Top,cmANDNOT,False);
					Act:=Act^.Prev;
				end;
			end;
			CanvasShape.SurfaceCopy(Canvas);
		end else
			SurfaceClear(False); {No Drawing!!!}
end;

procedure TView.MoveOffset(RelX,RelY: Integer);
begin
	with Canvas^.Offset do begin
		Inc(X,RelX);
		Inc(Y,RelY);
	end;
end;

procedure TView.RefreshCovered(X1,Y1,X2,Y2: Integer);
var
	Act: PView;
	I: Integer;
begin
	Act:=Next;
	if X1>X2 then begin
		I:=X1;
		X1:=X2;
		X2:=I;
	end;
	if Y1>Y2 then begin
		I:=Y1;
		Y1:=Y2;
		Y2:=I;
	end;
	while Assigned(Act) do
		with Act^ do begin
			if Visible and (Left<=X2) and (Top<=Y2) and (Left+Width>X1) and (Top+Height>Y1) then
				RefreshCanvas;
			Act:=Next;
		end;
end;

destructor TView.Done;
var
	Act,Temp: PTimer;
begin
{$IFDEF DEBUG}
	Dec(Views);
{$ENDIF}
	Hide;
	SetEventHandler(nil);
	Act:=TimerQueue;
	while Assigned(Act) do begin
		Temp:=Act^.Next;
		if Act^.ViewHandle=Handle then
			Act^.Free;
		Act:=Temp;
	end;
	RemoveFromGroup;
	CanvasShape.Done;
	if Screen^.ViewUnderMouse=@self then
		Screen^.ViewUnderMouse:=nil;
	ClearBackground;
	if Assigned(ColorSet) then
		ColorSet^.Free;
	inherited Done;
	Canvas^.Free;
end;

function TView.CanDraw: Boolean;
begin
	CanDraw:=Visible and (Canvas^.SurfaceCount>0);
end;

procedure TView.FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer);
begin
	if Assigned(Background) then begin
		Back:=Background;
		OffsetX:=0;
		OffsetY:=0;
	end else if Assigned(Owner) then begin
		Owner^.FindBackground(Back,OffsetX,OffsetY);
		Dec(OffsetX,Left);
		Dec(OffsetY,Top);
	end else
		Back:=nil;
end;

procedure TView.SetBackground(ABack: PBitmap);
begin
	if ABack<>Background then begin
		ClearBackground;
		Background:=ABack;
		Background^.Allocate;
		ReDraw(nil);
	end;
end;

procedure TView.ClearBackground;
begin
	if Assigned(Background) then begin
		Background^.Free;
		Background:=nil;
	end;
end;

procedure TView.ChangeColorSet(AColorset: PColorSet);
begin
	AColorSet^.Allocate;
	if Assigned(ColorSet) then
		ColorSet^.Free;
	ColorSet:=AColorSet;
	if CanDraw then
		Draw;
end;

procedure TView.UniqueColorSet;
begin
	ColorSet:=ColorSet^.Unique;
end;

procedure TView.SetEventHandler(AHandle: PEventHandle);
begin
	if Assigned(EventHandler) then
		EventHandler^.Free;
	if AHandle=Handle then
		RunError;
	EventHandler:=AHandle;
	if Assigned(EventHandler) then
		EventHandler^.Allocate;
end;

{* TViewGroup *}

constructor TViewGroup.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	IsGroup:=True;
	CreateChilds;
end;

procedure TViewGroup.CreateChilds;
begin
end;

procedure TViewGroup.HandleEvent(var Event: TEvent);
var
	Temp,Act: PView;
begin
	inherited HandleEvent(Event);
	if Event.What=evBroadCast then begin
		Act:=Child;
		while Assigned(Act) and (Event.What<>evNothing) do begin
			Temp:=Act;
			Act:=Act^.Next; {Temp may be released so Act needs to be already valid}
			Temp^.HandleEvent(Event);
		end;
	end;
end;

procedure TViewGroup.SetDesigner(ADesigner: PEventObject);
var
	Act: PView;
begin
	inherited SetDesigner(ADesigner);
	Act:=Child;
	while Assigned(Act) do begin
		Act^.SetDesigner(ADesigner);
		Act:=Act^.Next;
	end;
end;

procedure TViewGroup.MoveOffset(RelX,RelY: Integer);
var
	Act: PView;
begin
	inherited MoveOffset(RelX,RelY);
	Act:=Child;
	with Canvas^ do
		while Assigned(Act) do begin
			Act^.MoveOffset(RelX,RelY);
			Act:=Act^.Next;
		end;
end;

procedure TViewGroup.Resize(AWidth,AHeight: Integer);
var
	OldW,OldH,X,Y,W,H: Integer;
	ODraw,IDraw: TSurface;
	Act: PView;
begin
	if AWidth<SizeLimit.A.X then
		AWidth:=SizeLimit.A.X;
	if AHeight<SizeLimit.A.Y then
		AHeight:=SizeLimit.A.Y;
	if AWidth>SizeLimit.B.X then
		AWidth:=SizeLimit.B.X;
	if AHeight>SizeLimit.B.Y then
		AHeight:=SizeLimit.B.Y;
	if (AWidth<>Width) or (AHeight<>Height) then begin
		OldW:=Width;
		OldH:=Height;
		if Assigned(Owner) and Visible then begin
			ODraw.Init(Owner^.Width,Owner^.Height,False);
			ODraw.SurfaceCombine(@CanvasShape,Left,Top,cmOR,False);
			IDraw.Init(AWidth,AHeight,True);
			IDraw.SurfaceCombine(@self,0,0,cmANDNOT,False);
			SurfaceResize(AWidth,AHeight,True);
			RefreshCanvas;
			IDraw.SurfaceCombine(Canvas,0,0,cmOR,False);
			Canvas^.BeginUpdate;
			Redraw(@IDraw);
			IDraw.Done;
			ODraw.SurfaceCombine(@CanvasShape,Left,Top,cmANDNOT,False);
			Owner^.RefreshOwnCanvas;
			RefreshCovered(Left,Top,Left+Maximum(OldW,AWidth)-1,Top+Maximum(OldH,AHeight)-1);
			Owner^.Canvas^.BeginUpdate;
		end else begin
			SurfaceResize(AWidth,AHeight,True);
			ODraw.SurfaceCount:=-1; {hack to mark surface as non-allocated}
		end;
		Act:=Child;
		Dec(OldW,Width);
		Dec(OldH,Height);
		while Assigned(Act) do
			with Act^ do begin
				X:=Left;
				Y:=Top;
				W:=Width;
				H:=Height;
				if rfLeft in ResizeFlags then begin
					Dec(X,OldW);
					Inc(W,OldW);
				end;
				if rfTop in ResizeFlags then begin
					Dec(Y,OldH);
					Inc(H,OldH);
				end;
				if rfRight in ResizeFlags then
					Dec(W,OldW);
				if rfBottom in ResizeFlags then
					Dec(H,OldH);
				if (X<>Left) or (Y<>Top) then
					Move(X,Y);
				if (W<>Width) or (H<>Height) then
					Resize(W,H);
				Act:=Act^.Next;
			end;
		if ODraw.SurfaceCount>=0 then begin {do we have allocated ToDraw?}
			if ODraw.SurfaceCount>0 then
				Owner^.Redraw(@ODraw);
			Canvas^.EndUpdate;
			Owner^.Canvas^.EndUpdate;
			ODraw.Done;
		end;
	end;
end;

function TViewGroup.FindView(X,Y: Integer): PView;
var
	Act,Result: PView;
	TX,TY: Integer;
begin
	if Canvas^.PointIncluded(X,Y) then
		FindView:=@self
	else begin
		Act:=Child;
		Result:=nil;
		while Assigned(Act) do begin
			with Act^ do
				if Visible then begin
					TX:=X-Left;
					TY:=Y-Top;
					if (TX>=0) and (TY>=0) and (TX<Width) and (TY<Height) then
						Result:=Act^.FindView(TX,TY);
				end;
			if Assigned(Result) then begin
				FindView:=Result;
				Exit;
			end;
			Act:=Act^.Next;
		end;
		FindView:=nil;
	end;
end;

procedure TViewGroup.RefreshCanvas;
var
	Act: PView;
begin
	inherited RefreshCanvas;
	if Visible then
		with Canvas^ do begin
			Act:=Child;
			while Assigned(Act) do begin
				with Act^ do
					if Visible then begin
						RefreshCanvas;
						self.Canvas^.SurfaceCombine(Act,Left,Top,cmANDNOT,False);
					end;
				Act:=Act^.Next;
			end;
		end;
end;

procedure TViewGroup.RefreshOwnCanvas;
var
	Act: PView;
begin
	inherited RefreshCanvas;
	if Visible then
		with Canvas^ do begin
			Act:=Child;
			while Assigned(Act) do begin
				with Act^ do
					if Visible then
						self.Canvas^.SurfaceCombine(Act,Left,Top,cmANDNOT,False);
				Act:=Act^.Next;
			end;
		end;
end;

procedure TViewGroup.Redraw(ASurface: PSurface);
var
	Act: PView;
	Limit: TSurface;
begin
	if Visible then begin
		inherited Redraw(ASurface);
		if Assigned(Child) then
			if Assigned(ASurface) then begin
				Limit.Init(0,0,False);
				Act:=Child;
				Canvas^.BeginUpdate;
				repeat
					with Act^ do
						if Visible and ASurface^.RectOversect(Left,Top,Left+Width-1,Top+Height-1) then begin
							Limit.SurfaceCopy(@CanvasShape);
							Limit.SurfaceCombine(ASurface,-Left,-Top,cmAND,False);
							if Limit.SurfaceCount>0 then
								Redraw(@Limit);
						end;
					Act:=Act^.Next;
				until not Assigned(Act);
				Canvas^.EndUpdate;
				Limit.Done;
			end	else begin
				Act:=Child;
				Canvas^.BeginUpdate;
				repeat
					with Act^ do
						if Visible then
							Redraw(nil);
					Act:=Act^.Next;
				until not Assigned(Act);
				Canvas^.EndUpdate;
			end;
	end;
end;

function TViewGroup.InsertionGroup: PViewGroup;
begin
	InsertionGroup:=@self;
end;

destructor TViewGroup.Done;
var
	Act: PView;
begin
	Hide;
	while Assigned(Child) do
		Child^.Free;
	inherited Done;
end;

{* TScreen *}

constructor TScreen.Init(AWidth,AHeight: Integer);
begin
	Screen:=@self;
	inherited Init(nil);
	Focus:=@self;
	New(ColorSet,Init);
	with ColorSet^ do begin
		Frame.Up:=clWhite or $70000000;
		Frame.Down:=clDarkGray or $70000000;
		Frame.Flat:=(Frame.Up div 2 and $7F7F7F)+(Frame.Down div 2 and $7F7F7F);
		Normal.Text:=clBlack;
		Normal.Back:=clWhite;
		Selected.Text:=clWhite;
		Selected.Back:=clBlue;
		Disabled.Text:=clDarkGray;
		Disabled.Back:=clWhite;
		Hover.Text:=clBlack;
		Hover.Back:=Frame.Flat;
		Back:=clLightGray;
		Special:=$007580;
	end;
	Resize(AWidth,AHeight);
end;

procedure TScreen.SetFocus(AView: PView);
var
	Event: TEvent;
	Temp: PView;
begin
	Temp:=nil;
	while Assigned(AView) and (Temp<>AView) do begin
		Temp:=AView;
		while Assigned(AView) and (not (AView^.AcceptFocus and AView^.Visible)) do
			AView:=AView^.Owner;
		if Assigned(AView) then
			AView:=AView^.ViewToFocus;
	end;
	if (Temp=AView) and Assigned(AView) and (not (AView^.AcceptFocus and AView^.Visible)) then
		AView:=nil; {cannot focus -> deadloop}
	if Assigned(Modal) and (not AView^.IsModal) then begin
		Write(#7);
		Exit;
	end;
	if (Focus<>AView) and ((not (Assigned(AView)) or (AView^.AcceptFocus and AView^.Visible))) then begin
		OldFocus:=Focus;
		Focus:=AView;
		if not Assigned(Focus) then begin{no focus?}
			Focus:=Screen;
			AView:=Screen;
		end;
		while Assigned(AView^.Owner) do begin
			AView^.Owner^.FocusChild:=AView;
			AView:=AView^.Owner;
		end;
		with Event do begin
			Receiver:=nil;
			What:=evBroadcast;
			Command:=cmFocusChanged;
		end;
		HandleEvent(Event);
	end;
end;

function TScreen.NewCanvas: PCanvas;
begin
	Abstract;
end;

procedure TScreen.Update;
var
	Event: PEvent;
begin
	if (not Updating) then begin
		Updating:=True;
		Application^.Post.Mouse(evMouse,Mouse^.Position,Mouse^.Keys,False);
	end;
end;

procedure TScreen.HandleEvent(var Event: TEvent);
var
	Tmp,Tmp2: PView;
	TmpEvent: TEvent;
begin
	with Event do
		case What of
		evMouse,evMouseMove,evMouseDown,evMouseUp,evMouseDouble: begin
				if Assigned(Capture) then
					Tmp:=Capture
				else begin
					Tmp:=FindView(Where.X,Where.Y);
					if Tmp=nil then
						Exit;
					if Tmp<>ViewUnderMouse then begin
						Tmp2:=ViewUnderMouse;
						ViewUnderMouse:=Tmp;
						if Assigned(Tmp2) then
							Application^.Send.Command(Tmp2^.Handle,cmMouseLeave,0);
						Application^.Send.Command(ViewUnderMouse^.Handle,cmMouseEnter,0);
					end;
					if ((What=evMouseDown) or AutoFocus) and (Tmp<>Focus) then
						SetFocus(Tmp);
					Updating:=False;
				end;
				if (Tmp<>PView(@self)) and (What<>evMouse) and ((Tmp=Capture) or (not Assigned(Modal)) or Tmp^.IsModal) then
					with Tmp^,Canvas^.Offset do begin
						Dec(Where.X,X);
						Dec(Where.Y,Y);
						if (What=evMouseMove) and (Tmp=Capture) and Dragging then begin
							What:=evMouseDrag;
							Dec(Where.X,DraggingOffset.X);
							Dec(Where.Y,DraggingOffset.Y);
						end;
						if Assigned(Designer) and (Tmp<>Capture) then begin
							TmpEvent.What:=evDesign;
							TmpEvent.Receiver:=Tmp^.AllocateHandle;
							TmpEvent.Message:=@Event;
							Designer^.HandleEvent(TmpEvent);
							TmpEvent.Receiver^.Free;
						end else
							HandleEvent(Event);
					end;
				EventHandled(Event);
			end;
		evKeyboard: begin
				if Assigned(Capture) and (Capture<>PView(@self)) then
					Capture^.HandleEvent(Event)
				else if Assigned(Focus) and (Focus<>PView(@self)) then
					if Assigned(Focus^.Designer) then begin
						TmpEvent.What:=evDesign;
						TmpEvent.Receiver:=Focus^.AllocateHandle;
						TmpEvent.Message:=@Event;
						Focus^.Designer^.HandleEvent(TmpEvent);
						TmpEvent.Receiver^.Free;
					end else
						Focus^.HandleEvent(Event);
				if Event.What=evKeyboard then begin
					What:=evBroadcast;
					Command:=cmHandleKey;
					HandleEvent(Event);
				end;
				EventHandled(Event);
			end;
		else
			inherited HandleEvent(Event);
		end;
end;

procedure TScreen.ConvertBitmap(ABitmap: PBitmap);
begin
	ABitmap^.Convert(BitmapFormat);
end;

procedure TScreen.Draw;
var
	X,Y: Integer;
	Temp: TSurface;
begin
	if Assigned(Background) then
		if Tile then begin
			Y:=0;
			while Y<Height do begin
				X:=0;
				while X<Width do begin
					Canvas^.PutImage(Background,X,Y);
					Inc(X,Background^.Width);
				end;
				Inc(Y,Background^.Height);
			end;
		end else begin
			X:=(Width-Background^.Width) div 2;
			Y:=(Height-Background^.Height) div 2;
			Temp.InitCopy(Canvas);
			Temp.SurfaceBar(X,Y,X+Background^.Width-1,Y+Background^.Height-1,False);
			if Temp.SurfaceCount>0 then begin
				Temp.SurfaceSwap(Canvas);
				Canvas^.Clear;
				Temp.SurfaceSwap(Canvas);
			end;
			Temp.Done;
			Canvas^.PutImage(Background,X,Y);
		end
	else
		Canvas^.Clear;
end;

procedure TScreen.FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer);
begin
	Back:=nil;
end;

procedure TScreen.Hide;
begin
	Visible:=False;
	ReleaseCapture;
	Modal:=nil;
	SetFocus(@self);
end;

procedure TScreen.Show;
begin
	Visible:=True;
	RefreshCanvas;
	Redraw(nil);
end;

{* TMouse *}

constructor TMouse.Init;
begin
	inherited Init;
	Mouse:=@self;
end;

procedure TMouse.Show;
begin
	Abstract;
end;

procedure TMouse.Hide;
begin
	Abstract;
end;

procedure TMouse.Limit(X1,Y1,X2,Y2: Integer);
begin
	Abstract;
end;

procedure TMouse.Move(X,Y: Integer);
begin
	Position.X:=X;
	Position.Y:=Y;
end;

procedure TMouse.Update(X,Y,Key: Integer);
var
	APos: TPoint;
	I,J,K,Ev: Integer;
	L: Longint;
begin
	APos.X:=X;
	APos.Y:=Y;
	if Key<>Keys then begin
		L:=Timer;
		I:=Key xor Keys;
		Keys:=Key;
		J:=1;
		for K:=1 to 3 do begin
			if I and J<>0 then begin
				if Key and J>0 then begin
					Application^.Post.Mouse(evMouseDown,Position,J,False);
					if (L<=LastKeyTime+DoubleClick) and (LastKey=I) then
						Application^.Post.Mouse(evMouseDouble,Position,J,False);
					LastKey:=I;
					LastKeyTime:=L;
				end else
					Application^.Post.Mouse(evMouseUp,Position,J,False);
			end;
			J:=J shl 1;
		end;
	end;
	if (Position.X<>APos.X) or (Position.Y<>APos.Y) then begin
		Application^.Post.Mouse(evMouseMove,APos,Key,False);
		Position:=APos;
	end;
end;

procedure TMouse.IntUpdate(X,Y,Key: Integer);
begin
	Update(X,Y,Key);
end;

{* TEventPoster *}

procedure TEventPoster.OpenEvent(var Event: PEvent);
begin
	Event:=Application^.Events.Put;
end;

procedure TEventPoster.Mouse(Kind: Word; const AWhere: TPoint; AButtons: Byte; ADouble: Boolean);
var
	Event: PEvent;
begin
	OpenEvent(Event);
	with Event^ do begin
		Receiver:=nil;
		What:=Kind;
		Buttons:=AButtons;
		Double:=ADouble;
		Where:=AWhere;
	end;
	CloseEvent(Event);
end;

procedure TEventPoster.Keyboard(Kind: Word; ACharCode: Char; AScanCode: Byte);
var
	Event: PEvent;
begin
	OpenEvent(Event);
	with Event^ do begin
		Receiver:=nil;
		What:=Kind;
		CharCode:=ACharCode;
		ScanCode:=AScanCode;
	end;
	CloseEvent(Event);
end;

procedure TEventPoster.Timer(Delay: Longint; AHandle: PEventHandle; ACommand: Word; ATag: Longint);
var
	Act: PTimer;
begin
	if not Assigned(AHandle) then
		RunError;
	Act:=TimerQueue;
	while Assigned(Act) do
		with Act^ do begin
			if (ACommand=Command) and (ATag=Tag) and (AHandle=ViewHandle) then begin
				Free;
				Break;
			end;
			Act:=Next;
		end;
	if Delay>0 then
		ATag:=Longint(New(PTimer,Init(Delay,AHandle,ACommand,ATag)));
end;

constructor TTimer.Init(Delay: Longint; AHandle: PEventHandle; ACommand: Word; ATag: Longint);
var
	InsertTo: ^PTimer;
	Act: PTimer;
begin
	inherited Init;
	Inc(TimerCnt);
	Due:=Timer+Delay;
	ViewHandle:=AHandle;
	ViewHandle^.Allocate;
	Command:=ACommand;
	Tag:=ATag;
	InsertTo:=@TimerQueue;
	Act:=TimerQueue;
	while Assigned(Act) and (Act^.Due<Due) do begin
		InsertTo:=@Act^.Next;
		Act:=Act^.Next;
	end;
	Next:=InsertTo^;
	InsertTo^:=@self;
end;

destructor TTimer.Done;
var
	RemoveFrom: ^PTimer;
begin
	ViewHandle^.Free;
	RemoveFrom:=@TimerQueue;
	while RemoveFrom^<>@self do
		RemoveFrom:=@RemoveFrom^^.Next;
	if Assigned(RemoveFrom^) then
		RemoveFrom^:=RemoveFrom^^.Next;
	Dec(TimerCnt);
	inherited Done;
end;

{* TEventSender *}

procedure TEventSender.OpenEvent(var Event: PEvent);
begin
	New(Event);
end;

procedure TEventSender.CloseEvent(Event: PEvent);
begin
	if not Assigned(Event^.Receiver) then
		RunError(0); {Undeliverable event}
	Event^.Receiver^.Event(Event^);
	Event^.Receiver^.Free;
	Dispose(Event);
end;

{* TApplication *}

constructor TApplication.Init(ADriver: PDriver);
begin
	if Assigned(Application) then
		RunError(0);
	if Assigned(ADriver) then
		Driver:=ADriver;
	inherited Init;
	if not Assigned(Driver) then
		RunError(0);
	Application:=@self;
	Events.Init;
	Post.Init;
	Send.Init;
	SystemFont:=New(PFont8,Init);
	PFont8(SystemFont)^.GrabSystemFont;
	Driver^.Open;
	if not Assigned(BitmapProcessor) then
		New(BitmapProcessor,Init);
	if not (Assigned(Screen) and Assigned(Timer) and Assigned(Mouse) and Assigned(SystemFont) and (Screen^.BitmapFormat<>0)) then
		RunError;
	Screen^.SetEventHandler(Handle);
	Move(Output,Screen^.OriginalOutput,SizeOf(Output));
	Move(Screen^.Canvas^.Text,Output,SizeOf(Output));
	with TTextRec(Output) do
		BufPtr:=@Buffer;
end;

procedure TApplication.Idle;
begin
end;

procedure TApplication.Run;
begin
	Screen^.Show;
	Mouse^.Show;
	Running:=True;
	while Running do begin
		GenerateEvents;
		Idle;
	end;
	Screen^.Hide; {prevent further screen output}
end;

procedure TApplication.GenerateEvents;
var
	Event: PEvent;
	Now: Longint;
begin
	if Running then begin
		Driver^.Check;
		Now:=Timer;
		while Assigned(TimerQueue) do
			with TimerQueue^ do
				if Due<=Now then begin
					if ViewHandle^.Assigned then
						Post.Command(ViewHandle,Command,Tag);
					Free;
				end else
					Break;
		while (Events.Count>0) and Running do begin
			Event:=Events.Get;
			with Event^ do begin
				if Assigned(Receiver) then begin
					Receiver^.Event(Event^);
					Receiver^.Free;
				end else case What of
				evKeyboard,evKeyDown,evKeyUp,
				evMouse,evMouseMove,evMouseDown,evMouseUp,evMouseDouble: begin
						LastInputEvent:=Timer;
						Screen^.HandleEvent(Event^);
					end;
				evBroadcast: Screen^.HandleEvent(Event^);
				else
					RunError(0); {Undeliverable event}
				end;
			end;
		end;
	end;
end;

function TApplication.IdleTime: Longint;
begin
	IdleTime:=Timer-LastInputEvent;
end;

procedure TApplication.Halt;
begin
	Running:=False;
end;

destructor TApplication.Done;
begin
	while Assigned(TimerQueue) do
		TimerQueue^.Free;
	while (Events.Count>0) do
		with Events.Get^ do
			if Assigned(Receiver) then
				Receiver^.Free;
	Move(Screen^.OriginalOutput,Output,SizeOf(Output));
	Driver^.Close;
	Dispose(Mouse,Done);
	Dispose(Screen,Done);
	Events.Done;
	SystemFont^.Free;
	inherited Done;
	Driver^.Done;
	Application:=nil;
	Screen:=nil;
	Mouse:=nil;
	Timer:=nil;
end;

procedure TDriver.Open;
begin
	Abstract;
end;

procedure TDriver.Check;
begin
end;

procedure TDriver.Close;
begin
end;

end.